﻿'2011 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net

Imports System.Linq
Imports System.Reflection
Imports System.Runtime.Serialization
Imports System.Xml

Namespace ServiceModelEx
   Public Class GenericResolver
	   Inherits DataContractResolver
	  Private Const DefaultNamespace As String = "global"

	  Private ReadOnly m_TypeToNames As Dictionary(Of Type,Tuple(Of String,String))
	  Private ReadOnly m_NamesToType As Dictionary(Of String,Dictionary(Of String,Type))

	  Public ReadOnly Property KnownTypes() As Type()
		 Get
			Return m_TypeToNames.Keys.ToArray()
		 End Get
	  End Property

	  Public Sub New()
		  Me.New(ReflectTypes())
	  End Sub

	  Public Sub New(ByVal typesToResolve() As Type)
		 m_TypeToNames = New Dictionary(Of Type,Tuple(Of String,String))()
		 m_NamesToType = New Dictionary(Of String,Dictionary(Of String,Type))()

		 For Each type As Type In typesToResolve
                Dim typeNamespace = GetNamespace(type)
                Dim typeName = GetName(type)

			m_TypeToNames(type) = New Tuple(Of String,String)(typeNamespace,typeName)

                If m_NamesToType.ContainsKey(typeNamespace) = False Then
                    m_NamesToType(typeNamespace) = New Dictionary(Of String, Type)
                End If
                m_NamesToType(typeNamespace)(typeName) = type
            Next type
	  End Sub

        Public Shared Function Merge(ByVal resolver1 As GenericResolver,
                                     ByVal resolver2 As GenericResolver) As GenericResolver
            If resolver1 Is Nothing Then Return resolver2
            If resolver2 Is Nothing Then Return resolver1
            Dim types As New List(Of Type)()
            types.AddRange(resolver1.KnownTypes)
            types.AddRange(resolver2.KnownTypes)

            Return New GenericResolver(types.ToArray())
        End Function
	  Private Shared Function GetNamespace(ByVal type As Type) As String
		 Return If(type.Namespace, DefaultNamespace)
	  End Function

	  Private Shared Function GetName(ByVal type As Type) As String
		 Return type.Name
	  End Function

        Public Overrides Function ResolveName(ByVal typeName As String,
                                              ByVal typeNamespace As String,
                                              ByVal declaredType As Type,
                                              ByVal knownTypeResolver As DataContractResolver) As Type
            If m_NamesToType.ContainsKey(typeNamespace) Then
                If m_NamesToType(typeNamespace).ContainsKey(typeName) Then
                    Return m_NamesToType(typeNamespace)(typeName)
                End If
            End If
            Return knownTypeResolver.ResolveName(typeName, typeNamespace, declaredType, Nothing)
        End Function
        Public Overrides Function TryResolveType(ByVal type As Type,
                                                 ByVal declaredType As Type,
                                                 ByVal knownTypeResolver As DataContractResolver,
                                                 <System.Runtime.InteropServices.Out()> ByRef typeName As XmlDictionaryString,
                                                 <System.Runtime.InteropServices.Out()> ByRef typeNamespace As XmlDictionaryString) As Boolean
            If m_TypeToNames.ContainsKey(type) Then
                Dim dictionary As New XmlDictionary()
                typeNamespace = dictionary.Add(m_TypeToNames(type).Item1)
                typeName = dictionary.Add(m_TypeToNames(type).Item2)

                Return True
            Else
                Return knownTypeResolver.TryResolveType(type, declaredType, Nothing, typeName, typeNamespace)
            End If
        End Function

        'Shared helpers
	  Private Shared Function ReflectTypes() As Type()
            Dim assemblyReferecnes() = GetCustomReferecnedAssemblies()

		 Dim types As New List(Of Type)()

            For Each [assembly] In assemblyReferecnes
                Dim typesInReferencedAssembly() = GetTypes([assembly])
                types.AddRange(typesInReferencedAssembly)
            Next [assembly]
            Dim entryAssembly = System.Reflection.Assembly.GetEntryAssembly()
		 If entryAssembly Is Nothing Then
			entryAssembly = GenericResolverInstaller.CallingAssembly
			Debug.Assert(entryAssembly IsNot Nothing)
		 End If

            Dim typesInCallingAssembly() = GetTypes(entryAssembly, False)
		 types.AddRange(typesInCallingAssembly)

		 Return types.ToArray()
	  End Function

	  Private Shared Function GetCustomReferecnedAssemblies() As System.Reflection.Assembly()
            Dim [assembly] = GenericResolverInstaller.CallingAssembly

            Dim referencedAssemblies() = [assembly].GetReferencedAssemblies()

            Const dotNetKeyToken1 = "b77a5c561934e089"
            Const dotNetKeyToken2 = "b03f5f7f11d50a3a"
            Const dotNetKeyToken3 = "31bf3856ad364e35"

		 Dim assemblies As New List(Of System.Reflection.Assembly)()
            For Each assemblyName In referencedAssemblies
                Dim keyToken = assemblyName.FullName.Split("="c)(3)

                Dim serviceNModelEx = System.Reflection.Assembly.GetCallingAssembly()
                Dim serviceNModelExKeyToken = serviceNModelEx.GetName().FullName.Split("="c)(3)
                If keyToken = serviceNModelExKeyToken Then Continue For
                Select Case keyToken
                    Case dotNetKeyToken1, dotNetKeyToken2, dotNetKeyToken3
                        Continue For
                End Select
                assemblies.Add(System.Reflection.Assembly.Load(assemblyName))
            Next assemblyName
		 Return assemblies.ToArray()
	  End Function

        Private Shared Function GetTypes(ByVal [assembly] As System.Reflection.Assembly,
                                         Optional ByVal publicOnly As Boolean = True) As Type()
            Dim allTypes() = [assembly].GetTypes()

            Dim types As New List(Of Type)()

            For Each type In allTypes
                If type.IsEnum = False AndAlso
                    type.IsInterface = False AndAlso
                    type.IsPublic = publicOnly AndAlso
                    type.IsGenericTypeDefinition = False Then types.Add(type)
            Next type
            Return types.ToArray()
        End Function
   End Class
End Namespace