﻿'2011 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net

Imports System.Linq
Imports System.Reflection
Imports System.Runtime.CompilerServices
Imports System.ServiceModel
Imports System.ServiceModel.Description


Namespace ServiceModelEx
   Public Module GenericResolverInstaller
	  Friend CallingAssembly As System.Reflection.Assembly

        <MethodImpl(MethodImplOptions.NoInlining)>
        <System.Runtime.CompilerServices.Extension()>
        Public Sub AddGenericResolver(ByVal host As ServiceHost, ByVal ParamArray typesToResolve() As Type)
            If CallingAssembly Is Nothing Then
                CallingAssembly = If(System.Reflection.Assembly.GetEntryAssembly(), System.Reflection.Assembly.GetCallingAssembly())
                VerifyAssembly(CallingAssembly)
            End If
            Trace.WriteLine("Found assembly: " & CallingAssembly.FullName)

            Debug.Assert(host.State <> CommunicationState.Opened)

            For Each endpoint In host.Description.Endpoints
                AddGenericResolver(endpoint, typesToResolve)
            Next endpoint
        End Sub

        <MethodImpl(MethodImplOptions.NoInlining)>
        <System.Runtime.CompilerServices.Extension()>
        Public Sub AddGenericResolver(Of T As Class)(ByVal proxy As ClientBase(Of T), ByVal ParamArray typesToResolve() As Type)
            If CallingAssembly Is Nothing Then
                CallingAssembly = If(System.Reflection.Assembly.GetEntryAssembly(), System.Reflection.Assembly.GetCallingAssembly())
                VerifyAssembly(CallingAssembly)
            End If
            Trace.WriteLine("Found assembly: " & CallingAssembly.FullName)

            Debug.Assert(proxy.State <> CommunicationState.Opened)
            AddGenericResolver(proxy.Endpoint, typesToResolve)
        End Sub

        <MethodImpl(MethodImplOptions.NoInlining)>
        <System.Runtime.CompilerServices.Extension()>
        Public Sub AddGenericResolver(Of T As Class)(ByVal factory As ChannelFactory(Of T), ByVal ParamArray typesToResolve() As Type)
            If CallingAssembly Is Nothing Then
                CallingAssembly = If(System.Reflection.Assembly.GetEntryAssembly(), System.Reflection.Assembly.GetCallingAssembly())
                VerifyAssembly(CallingAssembly)
            End If

            Debug.Assert(factory.State <> CommunicationState.Opened)
            AddGenericResolver(factory.Endpoint, typesToResolve)
        End Sub

	  Private Sub VerifyAssembly(ByVal [assembly] As System.Reflection.Assembly)
		 If [assembly] Is Nothing Then Throw New InvalidOperationException()

		 If CallingAssembly.FullName = GetType(ServiceHost).Assembly.FullName Then Throw New InvalidOperationException()

		 If CallingAssembly.FullName = GetType(GenericResolverInstaller).Assembly.FullName Then Throw New InvalidOperationException()

            Trace.WriteLine("Found assembly: " + assembly.FullName)
        End Sub

	  Private Sub AddGenericResolver(ByVal endpoint As ServiceEndpoint, ByVal typesToResolve() As Type)
            For Each operation In endpoint.Contract.Operations
                Dim behavior = operation.Behaviors.Find(Of DataContractSerializerOperationBehavior)()
                Dim newResolver As GenericResolver

                If typesToResolve Is Nothing OrElse typesToResolve.Any().Equals(False) Then
                    newResolver = New GenericResolver()
                Else
                    newResolver = New GenericResolver(typesToResolve)
                End If

                Dim oldResolver = TryCast(behavior.DataContractResolver, GenericResolver)
                behavior.DataContractResolver = GenericResolver.Merge(oldResolver, newResolver)
            Next operation
	  End Sub
   End Module
End Namespace






